#!/usr/bin/env python
#coding=utf8
# Copyright (C) 2015, Alibaba Cloud Computing

#Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

#The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import os
import re
import time
import hashlib
from time import mktime
from optparse import OptionParser
from aliyun.sls.logitem import *
from aliyun.sls.slsclient import *
from aliyun.sls.putlogsrequest import *
try:
    from oss.oss_api import *
except:
    from oss_api import *
try:
    from oss.oss_xml_handler import *
except:
    from oss_xml_handler import *

oss_access_id = ""
oss_access_key = ""
sls_access_id = ""
sls_access_key = ""

def getscriptdir():
    return os.path.split( os.path.realpath( __file__ ) )[0]

def json_loads_byteified(json_text):
    return _byteify(
        json.loads(json_text, object_hook=_byteify),
        ignore_dicts=True
    )

def _byteify(data, ignore_dicts = False):
    # if this is a unicode string, return its string representation
    if isinstance(data, unicode):
        return data.encode('utf-8')
    # if this is a list of values, return list of byteified values
    if isinstance(data, list):
        return [ _byteify(item, ignore_dicts=True) for item in data ]
    # if this is a dictionary, return dictionary of byteified keys and values
    # but only if we haven't already byteified it
    if isinstance(data, dict) and not ignore_dicts:
        return {
            _byteify(key, ignore_dicts=True): _byteify(value, ignore_dicts=True)
            for key,value in data.iteritems()
        }
    # if it's anything else, return it in its original form
    return data

class OssHelper:

    def __init__(self,host,access_id,access_key):
        self.access_id = access_id
        self.access_key = access_key
        self.oss = OssAPI(host,access_id,access_key)

    def ListFile(self,bucket, prefix,beginStr,endStr):
        #列出bucket中所拥有的object
        marker = prefix+beginStr
        delimiter = ""
        maxkeys = "1000"
        headers = {}
        r = []
        endMarker = None
        if endStr is not None:
            endMarker = prefix+endStr
        while True:
            res = self.oss.get_bucket(bucket, prefix, marker, delimiter, maxkeys, headers)
            object = ""
            if (res.status / 100) == 2:
                body = res.read()
                h = GetBucketXml(body)
                (file_list, common_list) = h.list()
                if len(file_list) == 0:
                    break;
                for i in file_list:
                    if  endMarker is not None and i[0] >=endMarker:
                        break
                    if len(i[0]) > 0 and i[0][len(i[0])-1] != "/":
                        r.append(i[0])
                marker = file_list[len(file_list) -1][0]
            else:
                print "get bucket error", res.status
                break;
        return r

    def ParseLog(self,log,queue_name,topic_name):
        r = []
        obj = json_loads_byteified(log)
        if obj is None:
            return None
        for k in obj:
            msgId = k;
            v = obj[k]
            for log in v:
                if log.has_key("Time") :
                    t = log["Time"]
                    t = t[:19]
                    t = time.strptime(t,"%Y-%m-%d %H:%M:%S")
                    log["__time__"] = int(mktime(t));
                    log["messageId"] = msgId
                    if queue_name is not None:
                        log["QueueName"] = queue_name
                    elif topic_name is not None:
                        log["TopicName"] = topic_name
                    r.append(log)
        return r 

    def GetFileContentAsLogs(self,bucket,path,queue_name,topic_name):
        headers = {}
        res = self.oss.get_object(bucket, path, headers)
        get_buffer_size = 10*1024*1024
        last = ""
        logs = []
        if(res.status /100) == 2:
            while True:
                content = res.read(get_buffer_size)
                if content:
                    last = last+content
                else:
                    break
            log = self.ParseLog(last,queue_name,topic_name)
            if log is not None:
                logs.extend(log)
            else:
                print "error line",item
        return logs
    
class SlsHelper :
    def __init__(self,host,access_id,access_key,project,logstore,topic,quota):
        self.host = host
        self.access_id = access_id
        self.access_key = access_key
        self.sls_client = SLSClient(host,self.access_id,self.access_key)
        self.project = project;
        self.logstore = logstore
        self.topic=topic 
        self.quota = quota*1024*1024
        self.startTime =  time.time();
        self.usedQuota = 0
        if topic  is None:
            self.topic = "";

    def SendLogs(self,logs):
        if logs is None:
            return
        for i in range(len(logs)/1024 + 1):
            endRange = (i+1)*1024
            if endRange > len(logs):
                endRange = len(logs)
            items=[]
            for j in range(i*2014,endRange):
                if logs[j] is None:
                        break;
                item = LogItem(logs[j]["__time__"])
                for key in logs[j]:
                    if key != "__time__":
                        item.push_back(key,logs[j][key])
                items.append(item)
            req = PutLogsRequest(self.project,self.logstore,self.topic,"",items)
            now = time.time();
            curSize = sys.getsizeof(req)
            if now - self.startTime > 60:
                self.usedQuota = curSize;
                self.startTime = now;
            else:
                if self.usedQuota + curSize > self.quota :
                    print "sleep ",60-(now - self.startTime),"used quota",self.usedQuota,"cur size:",curSize
                    time.sleep(60-(now - self.startTime));
                    self.usedQuota = curSize;
                else:
                    self.usedQuota = self.usedQuota + curSize
            self.sls_client.put_logs(req)


if __name__ == "__main__": 
    parser = OptionParser()
    # options
    parser.add_option("", "--oss_access_id", dest="oss_access_id", help="oss access id,if not specified ,whill use access_id")
    parser.add_option("", "--oss_access_key", dest="oss_access_key", help="oss access key,if not specified ,whill use access_key")
    parser.add_option("", "--sls_access_id", dest="sls_access_id", help="sls access id,if not specified ,whill use access_id")
    parser.add_option("", "--sls_access_key", dest="sls_access_key", help="sls access key,if not specified ,whill use access_key")
    parser.add_option("", "--access_id", dest="access_id", help="access id,if not specified ,whill use oss_access_id/sls_access_id")
    parser.add_option("", "--access_key", dest="access_key", help="access key,if not specified ,whill use oss_access_key/sls_access_key")

    parser.add_option("", "--start_time", dest = "start_time",help="start time to download access_log,include startTime")
    parser.add_option("", "--end_time", dest = "end_time",help="end time to download access_log,exclusive endTime")
    parser.add_option("", "--log_bucket",dest="log_bucket",help="the LoggingBucket name, you can search from mns.console.aliyun.com")
    parser.add_option("","--oss_host",dest="oss_host",help="oss host name,e.g --oss_host=oss.aliyuncs.com or --oss_host=oss-cn-hangzhou.aliyuncs.com")

    parser.add_option("","--sls_host",dest="sls_host",help="sls host name,e.g --sls_host=sls-cn-hangzhou.aliyuncs.com")
    parser.add_option("","--sls_project",dest="sls_project",help="sls project  name,e.g --sls_project=xinhua_project")
    parser.add_option("","--sls_logstore",dest="sls_logstore",help="sls logstore name,e.g --sls_logstore=oss_access_log")
    parser.add_option("","--sls_topic",dest="sls_topic",help="sls logstore name,e.g --sls_topic=huabei")
    parser.add_option("","--sls_quota",dest="sls_quota",help="sls quota MB per minute,e.g.10M/minutes means --sls_quota=10",default="10",type="float");
    
    parser.add_option("","--queue_name",dest="queue_name",help="mns queue name , e.g. --queue_name=TestQueue")
    parser.add_option("","--topic_name",dest="topic_name",help="mns topic name , e.g. --topic_name=TestTopic")

    (option, args) = parser.parse_args()
    if option.access_id is not None  or  option.access_key is not None :
        oss_access_id = sls_access_id = option.access_id
        oss_access_key = sls_access_key = option.access_key
    else :
        oss_access_id = option.oss_access_id
        oss_access_key = option.oss_access_key
        sls_access_id = option.sls_access_id
        sls_access_key = option.sls_access_key

    if oss_access_id is None :
        print "Oss AccessId is not set,use --access_id=${} or --oss_access_id]${}"
        exit(0)
    if oss_access_key is None:
        print "Oss AccessKey is not set,use --access_key=${} or --oss_access_key]${}"
        exit(0)
    if sls_access_id is None:
        print "you have choose to send data to sls, sls access id must be set, use --access_id or --sls_access_id"
        exit(0)
    if sls_access_key is None:
        print "you have choose to send data to sls, sls access key must be set, use --access_key or --sls_access_key"
        exit(0)

    if option.oss_host is None:
        print "Oss Host is not set, e.g. --oss_host=oss.aliyuncs.com"
        exit(0)
    if option.log_bucket is None:
        print "--log_bucket must be set,it is the bucket name that store the log file "
        exit(0)

    if option.sls_project is None :
        print "you have choose to send data to sls, --sls_project must be set"
        exit(0)
    if option.sls_logstore is None:
        print "you have choose to send data to sls, --sls_logstore must be set"
        exit(0)
    if option.sls_host is None :
        print "you have choose to send data to sls, --sls_host must be set"
        exit(0)
    
    if (option.queue_name is None and  option.topic_name is None):
        print " you have to set --queue_name or --topic_name "
        exit(0)
    if (option.queue_name is not None and  option.topic_name is not  None):
        print " you can only set --queue_name or --topic_name ,cannot set both"
        exit(0)
    
    if( option.queue_name is not None):
        log_prefix = "QueueMessageLog/"+option.queue_name+"/"
    if (option.topic_name is not None):
        log_prefix = "TopicMessageLog/"+option.topic_name+"/"

    beginStr = option.start_time
    if beginStr is not None:
        if len(beginStr) > 8 :
            beginStr = beginStr[:8]+"/"+beginStr[8:]
    else:
        beginStr = ""
    
    endStr = option.end_time
    if endStr is not None:
        if len(endStr) > 8 :
            endStr = endStr[:8]+"/"+endStr[8:]


    ossObj = OssHelper(option.oss_host, oss_access_id, oss_access_key)
    slsObj = SlsHelper(option.sls_host, sls_access_id, sls_access_key, option.sls_project, option.sls_logstore, option.sls_topic, option.sls_quota)

    files = ossObj.ListFile(option.log_bucket, log_prefix, beginStr, endStr)
    if len(files) == 0:
        print "no log file found in oss://"+option.log_bucket+"/"+log_prefix

    for fIndex in range(len(files)):
        print files[fIndex],
        logs = ossObj.GetFileContentAsLogs(option.log_bucket, files[fIndex], option.queue_name, option.topic_name)

        print "----> sls ",option.sls_project,"(",option.sls_logstore,")",len(logs)," records"
        slsObj.SendLogs(logs)
